#!/usr/bin/perl

use strict;

use Ogg::Vorbis;

package OggToAu;

# This has some warts on it: The sampling rate is always assumed to be 44100, regardless
# of what sampling rate the ogg file has. The return code isn't very informative in case
# of an error. I'm also not sure whether the byte-swapping is always correct. It's
# necessary on my x86 machine, but I don't know what Ogg::Vorbis would actually do
# on a machine with the opposite byte order.

sub convert {

my $in_file = shift;
my $out_file = shift;

my $ogg = Ogg::Vorbis->new;
my $bitstream = 0;
my $pcm = '';
open(INPUT,"<$in_file") or return 0;
binmode(INPUT);
return 0 if ($ogg->open(*INPUT)<0);

my $buffer = '-' x 4096;
for (;;) {
  my $bytes = $ogg->read($buffer,4096,0,2,1,$bitstream);
  last if ($bytes<=0);
  $pcm = $pcm . substr($buffer,0,$bytes);
}
$ogg->clear;
close(INPUT);


my $header = 
    '.snd' 
  . pack(
    'N5',
    24,     # length of header (no information section)
    length($pcm),
    3,      # 16-bit linear PCM
    44100,  # sampling rate in Hz
    1,      # one channel (mono)
  );


open(OUTPUT,">$out_file") or return 0;
binmode(OUTPUT);
syswrite(OUTPUT, $header, length($header));
my $block_size = 1024;
for (my $i=0; $i<length($pcm); $i+=$block_size) {
  my $block = substr($pcm,$i,$block_size); # may actually end up being shorter than $block_size, at end of buffer
  my $swapped = pack('n*',unpack('v*',$block)); # This seems to be fairly efficient.
  syswrite(OUTPUT, $swapped, length($swapped));
}
close(OUTPUT);

return 1;

}

1;
